Stan Math Library: stan/math/prim/scal/prob/pareto_cdf_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

pareto_cdf_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_PARETO_CDF_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_PARETO_CDF_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_greater_or_equal.hpp>
9 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
10 #include <stan/math/prim/scal/err/check_not_nan.hpp>
11 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
12 #include <stan/math/prim/scal/fun/constants.hpp>
13 #include <stan/math/prim/scal/fun/multiply_log.hpp>
14 #include <stan/math/prim/scal/fun/value_of.hpp>
15 #include <boost/random/exponential_distribution.hpp>
16 #include <boost/random/variate_generator.hpp>
17 #include <cmath>
18 #include <limits>
19
20 namespace stan {
21 namespace math {
22
23 template <typename T_y, typename T_scale, typename T_shape>
24 typename return_type<T_y, T_scale, T_shape>::type
25 pareto_cdf_log(const T_y& y, const T_scale& y_min, const T_shape& alpha) {
26 typedef typename stan::partials_return_type<T_y, T_scale, T_shape>::type
27 T_partials_return;
28
29 // Size checks
30 if ( !( stan::length(y) && stan::length(y_min) && stan::length(alpha) ) )
31 return 0.0;
32
33 // Check errors
34 static const char* function("stan::math::pareto_cdf_log");
35
36 using stan::math::check_positive_finite;
37 using stan::math::check_not_nan;
38 using stan::math::check_greater_or_equal;
39 using stan::math::check_consistent_sizes;
40 using stan::math::check_nonnegative;
41 using stan::math::value_of;
42 using std::log;
43 using std::exp;
44
45 T_partials_return P(0.0);
46
47 check_not_nan(function, "Random variable", y);
48 check_nonnegative(function, "Random variable", y);
49 check_positive_finite(function, "Scale parameter", y_min);
50 check_positive_finite(function, "Shape parameter", alpha);
51 check_consistent_sizes(function,
52 "Random variable", y,
53 "Scale parameter", y_min,
54 "Shape parameter", alpha);
55
56 // Wrap arguments in vectors
57 VectorView<const T_y> y_vec(y);
58 VectorView<const T_scale> y_min_vec(y_min);
59 VectorView<const T_shape> alpha_vec(alpha);
60 size_t N = max_size(y, y_min, alpha);
61
62 OperandsAndPartials<T_y, T_scale, T_shape>
63 operands_and_partials(y, y_min, alpha);
64
65 // Explicit return for extreme values
66 // The gradients are technically ill-defined, but treated as zero
67
68 for (size_t i = 0; i < stan::length(y); i++) {
69 if (value_of(y_vec[i]) < value_of(y_min_vec[i]))
70 return operands_and_partials.value(stan::math::negative_infinity());
71 }
72
73 // Compute vectorized cdf_log and its gradients
74
75 for (size_t n = 0; n < N; n++) {
76 // Explicit results for extreme values
77 // The gradients are technically ill-defined, but treated as zero
78 if (value_of(y_vec[n]) == std::numeric_limits<double>::infinity()) {
79 return operands_and_partials.value(0.0);
80 }
81
82 // Pull out values
83 const T_partials_return log_dbl = log(value_of(y_min_vec[n])
84 / value_of(y_vec[n]));
85 const T_partials_return y_min_inv_dbl = 1.0 / value_of(y_min_vec[n]);
86 const T_partials_return alpha_dbl = value_of(alpha_vec[n]);
87
88 // Compute
89 const T_partials_return Pn = 1.0 - exp(alpha_dbl * log_dbl);
90
91 P += log(Pn);
92
93 if (!is_constant_struct<T_y>::value)
94 operands_and_partials.d_x1[n]
95 += alpha_dbl * y_min_inv_dbl * exp((alpha_dbl + 1) * log_dbl) / Pn;
96 if (!is_constant_struct<T_scale>::value)
97 operands_and_partials.d_x2[n]
98 -= alpha_dbl * y_min_inv_dbl * exp(alpha_dbl * log_dbl) / Pn;
99 if (!is_constant_struct<T_shape>::value)
100 operands_and_partials.d_x3[n]
101 -= exp(alpha_dbl * log_dbl) * log_dbl / Pn;
102 }
103
104 return operands_and_partials.value(P);
105 }
106 }
107 }
108 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::check_greater_or_equal
bool check_greater_or_equal(const char *function, const char *name, const T_y &y, const T_low &low)
Return true if y is greater or equal than low.
Definition: check_greater_or_equal.hpp:84
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
multiply_log.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::exp
fvar< T > exp(const fvar< T > &x)
Definition: exp.hpp:10
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
partials_return_type.hpp
OperandsAndPartials.hpp
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
stan::math::pareto_cdf_log
return_type< T_y, T_scale, T_shape >::type pareto_cdf_log(const T_y &y, const T_scale &y_min, const T_shape &alpha)
Definition: pareto_cdf_log.hpp:25
stan::math::check_nonnegative
bool check_nonnegative(const char *function, const char *name, const T_y &y)
Return true if y is non-negative.
Definition: check_nonnegative.hpp:66
check_greater_or_equal.hpp
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
stan::math::negative_infinity
double negative_infinity()
Return negative infinity.
Definition: constants.hpp:132

[ Stan Home Page ] © 2011–2016, Stan Development Team.